package SearchTrees;

public class SearchTree {
	SearchNode root;	// Wurzelknoten
	
	// Knostruktor: leerer Suchbaum
	SearchTree() {
		root = null;
	}
	
	public SearchNode Search (int k){
		SearchNode n = root;
		while (n != null && k != n.key)
		{
			if (k < n.key) n=n.left;
			else n = n.right;
		}
		return n;
	}
	
	public SearchNode SearchRecursive( int k ){		
		SearchNode result = SearchRecursion(root,k);
		return result;
	}
	
	private SearchNode SearchRecursion(SearchNode current_root, int k) {
		if (null != current_root && k != current_root.key){
			if (k < current_root.key) 
				return SearchRecursion(current_root.left, k);
			else
				return SearchRecursion(current_root.right, k);
		}
		else
			return current_root;		
	}
	
	
	
	public boolean HasKey(int k)
	{
		return (Search(k) != null);
	}
	
	public SearchNode Insert (int k) {
		if (root == null)
			return root = new SearchNode(k);  
		SearchNode t=root;
		while (true){
			if (k == t.key) return null;
			if (k < t.key){
				if (t.left == null)
					return t.left = new SearchNode(k);
				else 
					t = t.left;
			} 
			else { // k > t.key
				if (t.right == null)
					return t.right = new SearchNode(k);
				else 
					t = t.right;
			}
		}
	}
	
	public SearchNode SymmetricDesc(SearchNode node)
	{
		if (node.left == null) 
			return node.right;
		if (node.right == null) 
			return node.left;
		SearchNode n = node;
		SearchNode parent = null;
		n = n.right; // guaranteed to be non-null
		while (n.left != null)
		{
			parent = n;
			n = n.left;
		}
		// from here n.left = null;
		if (parent != null) // implies that n != node.right;
		{
			parent.left = n.right; // delete n from parent
			n.left = node.left; // set left child of n
			n.right = node.right; // set right child of n
		}
		else // n = node.right
		{
			n.left = node.left;
		}
		return n;
	}
	
	public boolean Delete (int k) {
		SearchNode n = root;
		if (n != null && n.key == k)
		{
			root = SymmetricDesc(root);
			return true;
		}
		else
		{
			while (n != null)
			{
				if (n.left != null && k == n.left.key)
				{
					n.left = SymmetricDesc(n.left);
					return true;
				}
				else if (n.right != null && k == n.right.key)
				{
					n.right = SymmetricDesc(n.right); 
					return true;
				}
				else if (k < n.key) n = n.left;
				else n = n.right;
			}
			return false;
		}
	}
			
	void spaces(int level)
	{
		for (int i = 0; i<level; ++i) System.out.print("-");
	}
	
	void print(SearchNode node, int level)
	{
		if (node == null) return;
		print(node.left,level+1);
		spaces(level); System.out.println(" "+node.key);
		print(node.right,level+1);
	}
	
	public void print()
	{
		print(root,0);
	}
		
	int GetDepth(SearchNode node)
	{
		if (node != null)
		{
			int left = GetDepth(node.left);
			int right = GetDepth(node.right);
			if (left > right) return left + 1;
			else return right + 1;
		}
		else
			return 0;
	}
	public int GetDepth()
	{
		return GetDepth(root);
	}
}
